import 'dart:async';

import 'package:daily_programmer_client_dart/app/base_command.dart';
import 'package:daily_programmer_client_dart/generator/project.dart';
import 'package:daily_programmer_client_dart/util/path_tool.dart';

/// Handles the `project` daily_programmer command.
class ProjectCommand extends BaseCommand {
  static const _dryRunFlag = 'dry-run';
  static const _authorOption = 'author';
  static const _emailOption = 'email';
  static const _nameOption = 'name';
  static const _pathOption = 'path';

  /// Handles the `project` command with flag.
  ProjectCommand() {
    argParser
      ..addFlag(_dryRunFlag,
          help: 'Don\'t generate, but lists files that were to be generated.')
      ..addOption(_nameOption,
          abbr: 'n',
          help: 'Class name of challenge.',
          defaultsTo: '0000_unknow_challenge')
      ..addOption(_pathOption,
          abbr: 'p',
          help: 'Project path, a new folder will be created unde this path for '
              'the challenge.',
          defaultsTo: '.')
      ..addOption(_authorOption,
          abbr: 'a', help: 'Author of project.', defaultsTo: 'Unknown Dev')
      ..addOption(_emailOption,
          abbr: 'e', help: 'Email of author.', defaultsTo: '');
  }

  @override
  String get description => 'Generate a new project challenge.';

  @override
  String get name => 'project';

  @override
  String get invocation =>
      '${BaseCommand.binaryName} $name <challenge_(name|number)> '
      '[--path <project/path>] [--name <Name>] [--author <Author>] '
      '[--email <email>] [--dry-run]';

  @override
  Future run() async => await new ProjectGenerator(
          readArgAsEntityName('Project name is needed.'),
          _projectPath,
          getEntityName(_name),
          _author,
          _email,
          dryRun: _dryRun)
      .generate();

  bool get _dryRun => argResults[_dryRunFlag];
  String get _author => argResults[_authorOption];
  String get _email => argResults[_emailOption];
  String get _name => argResults[_nameOption];
  String get _projectPath => getNormalizedPath(argResults[_pathOption]);
}
