import 'dart:io';

import 'package:daily_programmer_client_dart/util/ansicolor.dart';

import 'package:logging/logging.dart';

/// Set up logger for output messages to console.
class AppLogger {
  /// Logger initilised with the name of project.
  static AppLogger log = new AppLogger._('DailyPrograming');

  final Logger _logger;

  AppLogger._(String name) : _logger = new Logger(name) {
    final pens = {
      Level.FINE: new Ansi()..blue(),
      Level.INFO: new Ansi()..green(),
      Level.WARNING: new Ansi()..magenta(),
      Level.SEVERE: new Ansi()..red(bold: true)
    };

    Logger.root.onRecord.listen((record) {
      for (final line in record.message.split('\n')) {
        stderr.writeln('${pens[record.level]('${record.level.name}:')} $line');
      }
    });
  }

  /// Configures the log level.
  ///
  /// By default, FINE log will not be printed.
  /// Sets [isVerbose] to print FINE logs also.
  set isVerbose(bool value) =>
      Logger.root.level = value ? Level.FINE : Level.INFO;

  /// Log with FINE level.
  // ignore: type_annotate_public_apis
  void fine(message, [Object error, StackTrace stackTrace]) =>
      _logger.log(Level.FINE, message, error, stackTrace);

  /// Log with INFO level.
  // ignore: type_annotate_public_apis
  void info(message, [Object error, StackTrace stackTrace]) =>
      _logger.log(Level.INFO, message, error, stackTrace);

  /// Log with WARNING level.
  // ignore: type_annotate_public_apis
  void warning(message, [Object error, StackTrace stackTrace]) =>
      _logger.log(Level.WARNING, message, error, stackTrace);

  /// Log with SEVERE level.
  // ignore: type_annotate_public_apis
  void severe(message, [Object error, StackTrace stackTrace]) =>
      _logger.log(Level.SEVERE, message, error, stackTrace);
}
